@@ -118,7 +118,16 @@ function lowerbound(weights::AbstractVector, graph::AbstractGraph, ::MMW{S}) whe
 end
 
 function mmw(weights::AbstractVector{W}, graph::AbstractGraph{V}, strategy::Val) where {W, V}
-    width = mmw(trunc.(V, weights), graph, strategy)
+    @argcheck nv(graph) <= length(weights)
+
+    n = nv(graph)
+    intweights = FVector{V}(undef, n)
+
+    @inbounds for v in vertices(graph)
+        intweights[v] = trunc(V, weights[v])
+    end
+
+    width = mmw(intweights, graph, strategy)
     return convert(W, width)
 end
 
@@ -835,6 +844,9 @@ function mmw_reach!(
         # to `vv`
         pp = begptr[vv]; ppend = endptr[vv]
 
+        # `ppnxt` is the largest possible value of `ppend`
+        ppnxt = begptr[vv + one(V)]
+
         while pp < ppend
             # `ww` is adjacent to `vv` and reachable by `v`
             ww = target[pp]
@@ -846,9 +858,6 @@ function mmw_reach!(
 
             # otherwise, `ww` is a supernode
             else
-                # `ppnxt` is the largest possible value of `ppend`
-                ppnxt = begptr[vv + one(V)]
-
                 # the arcs {`qq`, ..., `qqend` - 1} are incident
                 # to `ww`
                 ww = -ww; qq = begptr[ww]; qqend = endptr[ww]
